library out_nextprogram;

(* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version 
 * 1.1 (the "License"); you may not use this file except in compliance with 
 * the License. You may obtain a copy of the License at 
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the NextProgram Output Module for Lightning.
 *
 * The Initial Developer of the Original Code is
 * Tomsoft.
 * Portions created by Tomsoft are Copyright (C) 2011
 * Tomsoft. All Rights Reserved.
 *
 * Contributor(s):
 *   Tommy Sools <tom@sools.com>
 *
 * ***** END LICENSE BLOCK ***** *)

{$IMAGEBASE $41240000}

uses
  Windows,
  SysUtils,
  Graphics,
  StdCtrls,
  common in '..\common.pas';

{$R *.RES}

type
  TDetails = class(TObject)
  public
    pulsed:    Boolean;
    index:     Integer;
    abs:       Boolean;
    gbDetails: TGroupBox;
    lOffset:   TLabel;
    eOffset:   TEdit;
    bOffset:   TButton;
    procedure bOffsetClick(Sender: TObject);
  end;

procedure TDetails.bOffsetClick(Sender: TObject);
var
  bString: string;
begin
  bString := eOffset.Text;
  if Length(bString) > 0 then
  begin
    if bString[1] in ['+', '-'] then
    begin
      abs := False;
      Delete(bString, 1, 1);
    end
    else
      abs := True;
    try
      index := StrToInt(bString);
    except
      on EConvertError do
      begin
        abs   := False;
        index := 0;
      end;
    end;
  end
  else
  begin
    abs   := False;
    index := 0;
  end;
end;

function init(name: PChar; nameLength: DWORD): DWORD; cdecl;
begin
  Result := return('ONextProgram|Scene Control', name, nameLength);
end;

procedure finish(); cdecl;
begin
  //
end;

function create(controlHandle: THandle): PLightningModule; cdecl;
var
  myDetails: TDetails;
begin
  New(Result);
  with Result^ do
  begin
    handle        := controlHandle;
    title         := 'NextProgram';
    settings      := True;
    width         := -1;
    height        := -1;
    inputTop      := -1;
    infoTop       := -1;
    outputTop     := -1;
    inputs        := 1;
    outputs       := 0;
    inputTooltip  := 'Load next program pulse';
    outputTooltip := '';
    myDetails     := TDetails.Create;
    details       := myDetails;
  end;
  with myDetails do
  begin
    pulsed := False;
    index  := 1;
    abs    := False;
  end;
  New(PLightningModule2(Result^.reserved));
  New(PLightningModule3(PLightningModule2(Result^.reserved)^.reserved));
end;

function  action(lightningModule: PLightningModule; input, output: Integer; inputConnected, outputConnected: PChar; var bridge: TSpectrum): Boolean; cdecl;
var
  details: TDetails;
begin
  details := TDetails(lightningModule^.details);
  if spectrumAverage(bridge, fftMin, fftMax) > enabledBoundary then
  begin
    if not(details.pulsed) then
    begin
      details.pulsed := True;
      PLightningModule3(PLightningModule2(lightningModule^.reserved)^.reserved)^.nextProgramEx(details.index, details.abs);
    end;
  end
  else
    details.pulsed := False;
  Result := False;
end;

procedure show(lightningModule: PLightningModule; handle: THandle); cdecl;
begin
  with TDetails(lightningModule.details) do
  begin
    gbDetails := TGroupBox.Create(nil);
    with gbDetails do
    begin
      Top          := detailsTop;
      Left         := detailsLeft;
      Width        := detailsWidth;
      Height       := detailsHeight;
      Ctl3D        := True;
      Color        := clBtnFace;
      Caption      := 'NextProgram Settings';
    end;
    lOffset := TLabel.Create(gbDetails);
    with lOffset do
    begin
      AutoSize   := True;
      Top        := 20;
      Left       := 8;
      Caption    := 'Program index or offset:';
      Parent     := gbDetails;
    end;
    eOffset := TEdit.Create(gbDetails);
    with eOffset do
    begin
      Top    := 40;
      Left   := 8;
      Width  := 120;
      if not(abs) and (index > 0) then
        Text := '+'
      else
        Text := '';
      Text   := Text + IntToStr(index);
      Parent := gbDetails;
    end;
    bOffset := TButton.Create(gbDetails);
    with bOffset do
    begin
      Top     := 40;
      Left    := 140;
      Width   := 120;
      Height  := eOffset.Height;
      Caption := 'Update';
      OnClick := bOffsetClick;
      Parent  := gbDetails;
    end;
    gbDetails.ParentWindow := handle;
  end;
end;

procedure hide(lightningModule: PLightningModule); cdecl;
begin
  with TDetails(lightningModule.details) do
  begin
    bOffset.Free;
    eOffset.Free;
    lOffset.Free;
    gbDetails.Free;
  end;
end;

procedure open(lightningModule: PLightningModule; settings: PChar); cdecl;
var
  bString: string;
begin
  bString := settings;
  with TDetails(lightningModule.details) do
  begin
    index := StrToIntDef(parse(bString, ','), 1);
    abs   := Trim(parse(bString, ',')) <> '';
  end;
end;

function  save(lightningModule: PLightningModule; settings: PChar; settingsLength: DWORD): DWORD; cdecl;
var
  bString: string;
begin
  with TDetails(lightningModule.details) do
  begin
    if abs then
      bString := '1'
    else
      bString := '';
    Result := return(IntToStr(index) + ',' + bString, settings, settingsLength);
  end;
end;

procedure destroy(lightningModule: PLightningModule); cdecl;
begin
  TDetails(lightningModule^.details).Free;
  Dispose(PLightningModule3(PLightningModule2(lightningModule^.reserved)^.reserved));
  Dispose(PLightningModule2(lightningModule^.reserved));
  Dispose(lightningModule);
end;

exports
  init, finish, create, action, show, hide, open, save, destroy;

end.
